import { Button, Col, Form, notification, Row } from "antd";
import React, { useContext, useEffect, useState } from "react";
import {
  base_url,
  create_event, get_event_by_id,
  get_related_info, update_event
} from "../apiEndpoints/endPoints";
import EventContext from "../context/EventContext";
import {
	getDataAttrForBuddyBoss,
  getEventData,
  openNotification,
  prepareEventDataForContext,
  removeEventDataFromEditMode,
  settingEventAllProp
} from "../helpers/helper";
import ConfirmModal from "./common/ConfirmModal";
import { stepsData } from "./common/Header";
import Spinner from "./common/Spinner";
import EventInfo from "./steps/EventInfo";
import GeneralInfo from "./steps/GeneralInfo";

const { __ } = wp.i18n;


function Wrapper({ shouldUpdate, event_id }) {
  const {
    nextStep,
    state,
    setCategoryData,
    setTagsData,
    setScheduleData,
    setOrganizerData,
    setTimeZoneData,
    onChangeFormData,
    setEventData,
  } = useContext(EventContext);
  const {
    step,
    isLoading,
    post_title,
    post_content,
    redirect_cancel_url,
    redirect_ok_url,
    events
  } = state;

  const [eventId, setEventId] = useState(null);
  const [redirectUrl, setRedirectUrl] = useState(null);

  const onFinish = (values) => { };
  const onFinishFailed = (errorInfo) => { };

  /**
   Handler for SUBMIT button click 
   @argument none
   @return void
  */
  const handleSubmit = () => {
    window.scrollTo({ top: 0, behavior: "smooth" });
    // cannot submit when post title or content is empty.
    if (post_title.trim() == '' || post_content.trim() == '') {
      return openNotification(
        notification,
        "error",
        __("Input missing!", 'eventin-pro'),
        __("Post title and content should not empty!", 'eventin-pro')
      );
    }

    // structure event data object like database (getEventData) helper function.
    const eventData = getEventData(state);
    onChangeFormData("isLoading", true);
    // calling create_event api to store data
    fetch(create_event, {
      method: "POST",
      body: JSON.stringify(eventData),
    })
      .then(async (res) => {
        onChangeFormData("isLoading", false);
        const data = await res.json();

        
        if (data.status_code == 1) {
          const eventsArray = prepareEventDataForContext({ events, event: data?.content?.event })
          onChangeFormData('events', eventsArray)
          onChangeFormData("success_title", __('Event Created.', 'eventin-pro'));
          onChangeFormData("success_msg", __('Your event has been created. Take next action from the below button.', 'eventin-pro'));
          onChangeFormData("show_modal", true);

        }
      })
      .catch((err) => {
        onChangeFormData("isLoading", false);
        openNotification(
          notification,
          "error",
          __("Event couldn't saved.", 'eventin-pro'),
          __("Your event is not saved successfully.", 'eventin-pro')
        );
      });
  };

  /**
    Handler for UPDATE button click 
    @param {String} id
    @return void
   */
  const handleUpdate = (id) => {
    window.scrollTo({ top: 0, behavior: "smooth" });

    // cannot submit when post title or content is empty.
    if (post_title.trim() == '' || post_content.trim() == '') {
      return openNotification(
        notification,
        "error",
        __("Input missing!", 'eventin-pro'),
        __("Post title and content should not empty!", 'eventin-pro'),

      );
    }

    // structure event data object like database (getEventData) helper function.
    const eventData = getEventData(state);

    onChangeFormData("isLoading", true);

    // calling update_event api to update event
    fetch(update_event, {
      method: "POST",
      body: JSON.stringify({ ...eventData, event_id: id }),
    })
      .then(async (res) => {
        onChangeFormData("isLoading", false);
        const data = await res.json();
        const eventsArray = prepareEventDataForContext({ events, event: data?.content?.event })
        onChangeFormData('events', eventsArray)
        
        if (data.status_code == 1) {
          onChangeFormData("success_title", __('Event Updated.', 'eventin-pro'));
          onChangeFormData("success_msg", __('Your event has been updated. Take next action from the below button.', 'eventin-pro'));
          onChangeFormData("show_modal", true);
        }
        

      })
      .catch((err) => {
        console.log("err: ", err);
        onChangeFormData("isLoading", false);
        openNotification(
          notification,
          "error",
          __("Event couldn't saved.", 'eventin-pro'),
          __("Your event is not saved successfully.", 'eventin-pro'),

        );
      });
  };

  useEffect(() => {
    
    //get event_id from the url(if any)
    const params = new Proxy(new URLSearchParams(window.location.search), {
      get: (searchParams, prop) => searchParams.get(prop),
    });
    let value = event_id ? event_id : params.event_id;
    setEventId(value);
    onChangeFormData("event_id", value);
    onChangeFormData("author_id", localized_data_obj.author_id);
    onChangeFormData("isLoading", true);

    // calling get_related_info api to get all other data related to event.
    fetch(get_related_info)
      .then(async (res) => {
        const data = await res.json();
        const {
          categories,
          schedules,
          speaker_categories,
          tags,
          timezone_markup,
        } = data.content;

        // setting all other data related to event to context.
        setCategoryData(categories);
        setTagsData(tags);
        setOrganizerData(speaker_categories);
        setScheduleData(schedules);
        setTimeZoneData(timezone_markup);

        return data.content;
      })
      .then((_res) => {
        if ((value && shouldUpdate !== false)) {
          // if value is something, event form will be a update form.
          onChangeFormData("isLoading", true);

          // calling get_event_by_id api to get a specific event to edit.
          fetch(get_event_by_id + value)
            .then(async (res) => {
              const post = await res.json();
              setEventData(post.content);

              // setting all event data seperatly as form input value in context.(settingEventAllProp helper function)
              settingEventAllProp(onChangeFormData, post.content, _res);
              onChangeFormData("isLoading", false);
            })
            .catch((err) => {
              onChangeFormData("isLoading", false);
            });
        } else {
          onChangeFormData("isLoading", false);
        }
      })
      .catch((err) => {
        onChangeFormData("isLoading", false);
      });

    return () => {
      removeEventDataFromEditMode(onChangeFormData);
      // onChangeFormData("component", "event");
      nextStep(0);
      onChangeFormData("event_id", null);
     };
  }, []);

  /**
   * handler for go to NEXT step.
   * @returns {void}
   */
  const gotoNext = () => {
    if (post_title.trim() == '' || post_content.trim() == '') {
      return openNotification(
        notification,
        "error",
        __("Input missing!", 'eventin-pro'),
        __("Post title and content should not empty!", 'eventin-pro')

      );
    }
    nextStep(+step + 1);
  };

  const handleOk = () => {
    const url = window.location.href.includes('/?event_id') ? `${window.location.origin}${window.location.pathname}`
     : null;
     if(url){
       window.location.href = url      
     }else{
      // removeEventDataFromEditMode(onChangeFormData)
      onChangeFormData("component", "event");
      onChangeFormData("event_id", null);
      onChangeFormData("show_modal", false);
      nextStep(0);
     }
    
  };


  const hideModal = () => {

    const url = window.location.href.includes('/?event_id') || window.location.href.includes('create_event') 
    ? `${window.location.origin}${window.location.pathname.replace('create_event', 'vendor_event')}` : null;
    if(url){      
      window.location.href = url;
    }else{
      // removeEventDataFromEditMode(onChangeFormData)
      onChangeFormData("component", "table");
      onChangeFormData("event_id", null);
      onChangeFormData("show_modal", false);
      nextStep(0);
    }
    

  };


  if (isLoading) {
    return (
      <Spinner
        tip={__("Loading...", 'eventin-pro')}
        msg={__("Please Wait.", 'eventin-pro')}
        desc={__("Loader will vanish after all data load.", 'eventin-pro')}
        type={"info"}
        size={"large"}
        wrapperClassName={"ent-mltv-spinner-wrapper"}
      />
    );
  }

  return (
    <>
      <Form
        name="basic"
        layout="vertical"
        labelCol={{
          span: 8,
        }}
        wrapperCol={{
          span: 16,
        }}
        initialValues={{
          remember: true,
        }}
        onFinish={onFinish}
        onFinishFailed={onFinishFailed}
        autoComplete="off"
      >
        <Row>
          <Col span={24}>
            {step == 0 && <GeneralInfo />}
            {step == 1 && <EventInfo />}
            <ConfirmModal handleOk={handleOk} hideModal={hideModal} okText={"Create another"} cancelText={'Back to events'} icon={true} />
          </Col>
        </Row>

        <Row className="etn-mltv-form-action-wrap">
          <Col span={24}>
            {step > 0 ? (
              <Button
                className="etn-btn-default"
                type="default"
                size={"large"}
                onClick={() => {
                  nextStep(+step - 1);
                }}
              >
                {__('Previous', 'eventin-pro')}
              </Button>
            ) : null}
            {stepsData.length - 1 > step ? (
              <Button
                className="etn-btn-primary"
                type="primary"
                size={"large"}
                onClick={gotoNext}
              >
                {__('Next', 'eventin-pro')}
              </Button>
            ) : (
              <>
                {eventId && shouldUpdate !== false ? (
                  <Button
                    className="etn-btn-primary"
                    type="primary"
                    htmlType="submit"
                    size={"large"}
                    onClick={() => {
                      handleUpdate(eventId);
                    }}
                    disabled={isLoading}
                  >
                    {__('Update', 'eventin-pro')}
                  </Button>

                ) : (
                  <Button
                    className="etn-btn-primary"
                    type="primary"
                    htmlType="submit"
                    size={"large"}
                    onClick={handleSubmit}
                    disabled={isLoading}
                  >
                    {__('Submit', 'eventin-pro')}
                  </Button>
                )}
              </>
            )}
          </Col>
        </Row>
      </Form>
    </>
  );
}

export default Wrapper;
